from pathlib import Path
from housaku.files import list_files, read_txt, read_generic_doc

TEST_FILES_DIR = Path(__file__).parent / "examples"


def test_list_files():
    file_list = list_files(TEST_FILES_DIR)
    assert len(file_list) == sum(
        1 for item in TEST_FILES_DIR.iterdir() if item.is_file()
    )


def test_read_txt():
    for test_file in TEST_FILES_DIR.glob("*.txt"):
        uri, name, content = read_txt(test_file)
        assert uri == f"{test_file.resolve()}"
        assert name == test_file.name
        assert content


def test_read_pdf():
    for test_file in TEST_FILES_DIR.glob("*.pdf"):
        uri, name, content = read_generic_doc(test_file)
        assert uri == f"{test_file.resolve()}"
        assert name == test_file.name
        assert content


def test_read_epub():
    for test_file in TEST_FILES_DIR.glob("*.epub"):
        uri, name, content = read_generic_doc(test_file)
        assert uri == f"{test_file.resolve()}"
        assert name == test_file.name
        assert content


def test_read_docx():
    for test_file in TEST_FILES_DIR.glob("*.docx"):
        uri, name, content = read_generic_doc(test_file)
        assert uri == f"{test_file.resolve()}"
        assert name == test_file.name
        assert content


def test_bench_list_files(benchmark):
    file_list = benchmark(list_files, TEST_FILES_DIR)
    assert file_list


def test_bench_read_txt(benchmark):
    test_file = TEST_FILES_DIR / "gutenberg_moby_dick.txt"
    uri, _, _ = benchmark(read_txt, test_file)
    assert uri


def test_bench_read_pdf(benchmark):
    test_file = TEST_FILES_DIR / "gutenberg_the_modern_prometheus.pdf"
    uri, _, _ = benchmark(read_generic_doc, test_file)
    assert uri


def test_bench_read_epub(benchmark):
    test_file = TEST_FILES_DIR / "fundamental_accessibility.epub"
    uri, _, _ = benchmark(read_generic_doc, test_file)
    assert uri



from collections import deque
import fnmatch
import mimetypes
from pathlib import Path
import pymupdf
from housaku.models import Doc
from housaku.db import db_connection
from housaku.utils import console

GENERIC_FILETYPES = [
    "application/pdf",
    "application/epub+zip",
    "application/vnd.openxmlformats-officedocument.wordprocessingml.document",
    "application/vnd.openxmlformats-officedocument.presentationml.presentation",
    "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet",
]

pymupdf.JM_mupdf_show_errors = 0


def list_files(root: Path, exclude: list[str] = []) -> list[Path]:
    exclude_set = set(exclude)
    pending_dirs = deque([root])
    files = []

    if root.is_dir():
        while pending_dirs:
            dir = pending_dirs.popleft()
            for path in dir.iterdir():
                if any(fnmatch.fnmatch(path.name, pattern) for pattern in exclude_set):
                    continue

                if path.is_dir():
                    pending_dirs.append(path)

                if path.is_file():
                    files.append(path.resolve())
    else:
        if not any(fnmatch.fnmatch(root.name, pattern) for pattern in exclude_set):
            files.append(root.resolve())

    return files


def read_file(file: Path) -> Doc:
    mime_type, _ = mimetypes.guess_type(file)

    if mime_type == "text/plain":
        uri, title, content = read_txt(file)
    elif mime_type == "text/markdown":
        uri, title, content = read_txt(file)
    elif mime_type in GENERIC_FILETYPES:
        uri, title, content = read_generic_doc(file)
    else:
        raise Exception(f"Unsupported file format {mime_type}")

    return Doc(
        uri=f"{uri}",
        title=title,
        doc_type=mime_type,
        content=content,
    )


def read_txt(file: Path) -> tuple[str, str, str]:
    with open(file, "r") as f:
        return f"{file.resolve()}", file.name, f.read()


def read_generic_doc(file: Path) -> tuple[str, str, str]:
    content = ""
    with pymupdf.open(file) as doc:
        for page in doc:
            content += page.get_text()

    return f"{file.resolve()}", file.name, content


def index_file(sqlite_url: str, file: Path) -> None:
    try:
        doc = read_file(file)
        with db_connection(sqlite_url) as conn:
            cursor = conn.cursor()
            cursor.execute(
                """
            INSERT INTO documents (uri, title, type, content)
            VALUES (?, ?, ?, ?)
            """,
                (doc.uri, doc.title, doc.doc_type, doc.content),
            )
        console.print(f"[green][Ok][/] indexed '{file}'.")
    except Exception as e:
        console.print(f"[red][Err][/] something went wrong while reading '{file}': {e}")



from typing import Type, Tuple
from pathlib import Path
import shutil
import click
from pydantic import BaseModel, DirectoryPath
from pydantic_settings import (
    BaseSettings,
    PydanticBaseSettingsSource,
    SettingsConfigDict,
    TomlConfigSettingsSource,
)

app_dir = Path(click.get_app_dir(app_name="housaku"))
config_file_path = app_dir / "config.toml"
template_file_path = Path(__file__).parent / "config_template.toml"


class Files(BaseModel):
    include: list[DirectoryPath] = []
    exclude: list[str] = []


class Feeds(BaseModel):
    urls: list[str] = []


class Settings(BaseSettings):
    sqlite_url: str = f"{app_dir / 'db.sqlite3'}"
    files: Files = Files()
    feeds: Feeds = Feeds()

    model_config = SettingsConfigDict(toml_file=config_file_path)

    @classmethod
    def settings_customise_sources(
        cls,
        settings_cls: Type[BaseSettings],
        init_settings: PydanticBaseSettingsSource,
        env_settings: PydanticBaseSettingsSource,
        dotenv_settings: PydanticBaseSettingsSource,
        file_secret_settings: PydanticBaseSettingsSource,
    ) -> Tuple[PydanticBaseSettingsSource, ...]:
        app_dir.mkdir(parents=True, exist_ok=True)

        if not config_file_path.exists():
            shutil.copy(template_file_path, config_file_path)

        return (TomlConfigSettingsSource(settings_cls),)



from pathlib import Path
from typing import Annotated
from fastapi import FastAPI, Request, Form
from fastapi.staticfiles import StaticFiles
from fastapi.responses import HTMLResponse
from fastapi.templating import Jinja2Templates
from housaku.db import init_db
from housaku.settings import Settings
from housaku.search import search
from housaku.utils import console

settings = Settings()
init_db(settings.sqlite_url)

app = FastAPI()

base_dir = Path(__file__).resolve().parent.parent.parent
static_folder = base_dir / "static"
app.mount("/static", StaticFiles(directory=static_folder), name="static")
templates = Jinja2Templates(directory="templates")


@app.get("/", response_class=HTMLResponse)
async def index(request: Request):
    return templates.TemplateResponse(request=request, name="index.html", context={})


@app.post("/search/", response_class=HTMLResponse)
async def search_results(request: Request, query: Annotated[str, Form()]):
    try:
        results = search(settings.sqlite_url, query)
    except Exception as e:
        return 404, {"detail": f"{e}"}

    console.log(results)

    return templates.TemplateResponse(
        request=request,
        name="results.html",
        context={"query": query, "results": results},
    )



class Doc:
    def __init__(
        self,
        uri: str,
        title: str,
        doc_type: str,
        content: str,
    ) -> None:
        self.uri = uri
        self.title = title
        self.doc_type = doc_type
        self.content = content
